package com.link.core.cllog;

import com.link.core.kafka.KafkaTemplateFacotry;
import com.link.core.kafka.conf.KafkaEnv;
import java.text.SimpleDateFormat;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class LogMessageBuilder {
   private static final Logger logger = LogManager.getLogger(LogMessageBuilder.class);
   private static final String SEPARATOR = "^";
   private static final SimpleDateFormat DF = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss Z");
   private static final String IP = LCLogger.getIp();
   private Long id;
   private String level;
   private String className;
   private String messageKey;
   private StringBuilder message = new StringBuilder();

   private static String formatDate(Long currentTimeMillis) {
      synchronized(DF) {
         return DF.format(currentTimeMillis);
      }
   }

   LogMessageBuilder(String level, String className, Long id) {
      this.level = level;
      this.className = className;
      this.id = id;
   }

   public LogMessageBuilder withMessageKey(String messageKey) {
      this.messageKey = messageKey;
      return this;
   }

   public LogMessageBuilder withMessage(String message) {
      if (message == null) {
         message = "null";
      }

      this.message.append(this.mergeLines(message));
      return this;
   }

   public void flush() {
      if ("error".equals(this.level)) {
         logger.error("className:" + this.className + "\tmessageKey:" + this.messageKey + "\tmessage:" + this.message.toString());
      } else if ("warn".equals(this.level)) {
         logger.warn("className:" + this.className + "\tmessageKey:" + this.messageKey + "\tmessage:" + this.message.toString());
      } else {
         logger.info("className:" + this.className + "\tmessageKey:" + this.messageKey + "\tmessage:" + this.message.toString());
      }

      KafkaTemplateFacotry.send("KT_LOG", this.messageStream());
   }

   private String mergeLines(String value) {
      return value.replaceAll("\\\n", ";;;").replaceAll("=", ".eq.").replaceAll("\\|\\|", "\\\t");
   }

   private String messageStream() {
      String userIdAndName = LCLogger.getUserIdAndName();
      return formatDate(System.currentTimeMillis()) + "^" + handleNull(IP) + "^" + this.level + "^" + this.id + "^" + handleNull(this.className) + "^" + handleNull(this.messageKey) + "^" + handleNull(this.message.toString()) + "^" + "--" + "^" + "--" + "^" + handleNull(KafkaEnv.getProfile()) + "^" + userIdAndName;
   }

   public static boolean isNull(String str) {
      return null == str || "".equals(str);
   }

   public static String handleNull(String str) {
      return isNull(str) ? "--" : str;
   }
}
